@@ -1,4 +1,4 @@
-.schema project
+.dump project
 .dump element_type
 .dump list_item
 .schema ticket
