@@ -2,14 +2,28 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
+#include <sys/time.h>
 #include <cgic.h>
 #include <iconv.h>
+#include <errno.h>
 #include "util.h"
 #include "hook.h"
 
+/* プロジェクト名 */
+char g_project_name[DEFAULT_LENGTH] = "";
+char g_project_name_4_url[DEFAULT_LENGTH] = "";
+/* アクション名 */
+char g_action_name[DEFAULT_LENGTH] = "";
+/* PATH_INFO */
+char g_path_info[DEFAULT_LENGTH] = "";
+
 unsigned long url_encode(unsigned char*, unsigned char*, unsigned long);
 static Action* get_actions();
 
+void redirect_raw(const char*);
+void page_not_found();
+
 Action* actions = NULL;
 int alloc_count = 0;
 
@@ -29,6 +43,7 @@ void xfree(void* p)
 /*     d("xfree: %p\n", p); */
     alloc_count--;
     free(p);
+    p = NULL;
 }
 static Action* get_actions()
 {
@@ -61,26 +76,97 @@ void free_action_actions()
     d("alloc_count: %d\n", alloc_count);
 }
 
-void exec_action()
+/* 復帰できないエラーが発生した場合にエラーページを表示する。 */
+void print_error_page(char* file_name, int line_number, char* message)
+{
+    o("Status: 500 Starbug1 Internal Error.\r\n");
+    o("Content-Type: text/html\r\n\r\n");
+    o(  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
+        "<html xml:lang=\"ja\" lang=\"ja\" xmlns=\"http://www.w3.org/1999/xhtml\">"
+        "<head>"
+            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
+        "</head>"
+        "<body>"
+            "<h2>ERROR oops!</h2>"
+            "エラーが発生しましたm(_ _)m"
+            "<h3>detail information</h3>");
+    o(      "<p>[%s(%d)] %s</p>",
+                file_name,
+                line_number,
+                message);
+    o(      "<hr />"
+            "<div align=\"right\">"
+            "<address>Copyright smeghead 2007 - 2008.</address>"
+            "</div>"
+        "</body>"
+        "</html>");
+}
+
+ActionType analysis_action()
 {
     char* index;
+    char* script_name = cgiScriptName;
     char* path_info = cgiPathInfo;
-    char action_name[1024];
-    Action* a;
-    if (strlen(path_info) > 1) {
-        strncpy(action_name, path_info + 1, 1024);
-    } else {
-        strcpy(action_name, "");
-    }
-    if ((index = strchr(action_name, '/'))) {
-        *index = '\0';
-    }
+    char index_cgi_file_name[DEFAULT_LENGTH] = "";
+    char admin_cgi_file_name[DEFAULT_LENGTH] = "";
+    ActionType ret = ACTION_TYPE_NONE;
 
+    memset(g_project_name, 0, DEFAULT_LENGTH);
+    memset(g_action_name, 0, DEFAULT_LENGTH);
+    memset(g_path_info, 0, DEFAULT_LENGTH);
+    d("************************************\n");
+    d("REQUEST: %s%s\n", script_name, path_info);
+    if (strlen(path_info) > 1) {
+        strncpy(g_project_name, path_info + 1, DEFAULT_LENGTH);
+    } else {
+        redirect_raw("top");
+        die("redirect to top.");
+    }
+    if ((index = strchr(g_project_name, '/'))) {
+        *index = '\0';
+        strncpy(g_action_name, index + 1, DEFAULT_LENGTH - (strlen(g_project_name) + 1));
+        if ((index = strchr(g_action_name, '/'))) {
+            *index = '\0';
+            /* 残りをpath_infoとする */
+            strcpy(g_path_info, index + 1);
+        }
+    }
+    /* URLとして使われるプロジェクトID。URL encodeする。 */
+    url_encode((unsigned char*)g_project_name, (unsigned char*)g_project_name_4_url, DEFAULT_LENGTH);
+
+    sprintf(index_cgi_file_name, "index.%s", get_ext(cgiScriptName));
+    sprintf(admin_cgi_file_name, "admin.%s", get_ext(cgiScriptName));
+    /* 実行するモードの判定 */
+    if (strstr(script_name, index_cgi_file_name)) {
+        if (strcmp(g_project_name, "top") == 0) {
+            d("actiontype top\n");
+            ret = ACTION_TYPE_INDEX_TOP;
+        } else {
+            d("actiontype project\n");
+            ret = ACTION_TYPE_INDEX;
+        }
+    } else if (strstr(script_name, admin_cgi_file_name)) {
+        d("g_project_name: %s\n", g_project_name);
+        if (strcmp(g_project_name, "top") == 0) {
+            d("actiontype admin top\n");
+            ret = ACTION_TYPE_ADMIN_TOP;
+        } else {
+            d("actiontype admin project\n");
+            ret = ACTION_TYPE_ADMIN;
+        }
+    }
+    return ret;
+}
+void exec_action()
+{
+    Action* a;
+    char action_name[DEFAULT_LENGTH];
+    strcpy(action_name, g_action_name);
     for (a = get_actions(); a != NULL; a = a->next) {
         if (!strcmp(action_name, a->action_name)) {
-            d("exec_action start: %s\n", a->action_name);
+            d("=exec_action start: %s\n", a->action_name);
             a->action_func();
-            d("exec_action end  : %s\n", a->action_name);
+            d("=exec_action end  : %s\n", a->action_name);
             return;
         }
     }
@@ -90,8 +176,10 @@ void exec_action()
     } else {
         /* path_infoが空なら、top_actionを呼び出す。 */
         for (a = get_actions(); a != NULL; a = a->next) {
-            if (!strcmp("top", a->action_name)) {
+            if (!strcmp("top", a->action_name) || !strcmp("top_top", a->action_name)) {
+                d("=exec_action default_action start: %s\n", a->action_name);
                 a->action_func();
+                d("=exec_action default_action end  : %s\n", a->action_name);
                 return;
             }
         }
@@ -103,19 +191,78 @@ void exec_action()
             return cgiFormIO; \
         } \
     } 
-int get_ticket_syntax_len(char* data, size_t len)
+static const char PROJECT_SPLITTER = ':';
+
+int get_ticket_syntax_len(char* data, size_t len, char* project_id, char* ticket_id)
 {
-    int ticket_syntax_len = 0;
-    if (len == 0) return ticket_syntax_len; /* 最後の文字だった */
-    data += 1; /* 1文字進める。 */
-    while (len--) {
-        if (*data < '0' || *data > '9') 
-            break; /* 数字でない場合は、breakする。 */
-        data++;
-        ticket_syntax_len++;
+    char string[1024];
+    int index = 0;
+    char* block_p;
+    char* line_p;
+
+    memset(string, '\0', 1024);
+    strncpy(string, data, MIN(len, 1024 - 1));
+    if (len == 0) return index; /* 最後の文字だった */
+
+    block_p = strchr(string, ' ');
+    line_p = strchr(string, '\n');
+    if (!block_p)
+        block_p = string + strlen(string);
+    if (!line_p)
+        line_p = string + strlen(string);
+
+    /* 解析対象の文字列を特定 */
+    string[MIN(block_p - string, line_p - string)] = '\0';
+
+    {
+        char* project_mode = strchr(string, PROJECT_SPLITTER);
+        char* project_id_p = string + 1;
+        char* ticket_no_p = string + 1;
+
+        while (index++, 1) { /* each char */
+            /* 
+             *              * 一文字つづ進めていく。
+             *                           * 途中でチケットリンクでないと判定された場合は、index を 0 にして、breakする。
+             *                                        * チケットリンクが終了したら、breakする。
+             *                                                     */
+            if (string[index] == '\0') {
+                break;
+            } else if (project_mode) {
+                /* #プロジェクト:チケットモード */
+                if (string[index] == PROJECT_SPLITTER) {
+                    /* プロジェクトIDモードであり、:を検出した場合 */
+                    ticket_no_p = &string[index + 1];
+                    /* :で終了している場合は、チケットリンクではない。 */
+                    if (string[index + 1] < '0' || string[index + 1] > '9') {
+                        index = 0; break;
+                    }
+                } else if (project_id_p != ticket_no_p && (string[index] < '0' || string[index] > '9')) {
+                    /* チケットNO検索中に、数字以外を検出した場合 */
+                    string[index] = '\0'; break; /* 一つ前までがチケットリンクとなる。 */
+                }
+            } else if (!project_mode) {
+                /* #チケットモード */
+                if (string[index] >= '0' && string[index] <= '9') {
+                    /* プロジェクトIDモードではなく、数字を検出した場合 (正常) */
+                } else {
+                    /* reached to end. */
+                    string[index] = '\0'; break; /* 一つ前までがチケットリンクとなる。 */
+                }
+            }
+        }
+        if (index < 2) {
+            /* チケットリンクではない。 */
+            return 0;
+        }
+        if (project_id_p != ticket_no_p) {
+            strncpy(project_id, project_id_p, (ticket_no_p - 1) - project_id_p);
+        }
+        strcpy(ticket_id, ticket_no_p);
+        /*         index = (ticket_no_p - string) + strlen(ticket_no_p); */
     }
-    return ticket_syntax_len;
+    return index;
 }
+
 /*
  * 複数行テキストの領域では、pre記法をサポートする。
  * チケットリンクをサポートする。
@@ -126,16 +273,30 @@ static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
     while (len--) {
         if (*data == '#') {
             int ticket_syntax_len;
-            ticket_syntax_len = get_ticket_syntax_len(data, len);
+            char project_id[DEFAULT_LENGTH];
+            char ticket_id[DEFAULT_LENGTH];
+            memset(project_id, '\0', DEFAULT_LENGTH);
+            memset(ticket_id, '\0', DEFAULT_LENGTH);
+            /* ループの最初でlenが減っているので、get_ticket_syntax_lenに渡すときは戻して呼び出す。 */
+            ticket_syntax_len = get_ticket_syntax_len(data, len + 1, project_id, ticket_id);
+            d("ticket_syntax_len: %d \n", ticket_syntax_len);
             if (ticket_syntax_len == 0) {
                 TRYPUTC(*data);
             } else {
-                char ticket_id[DEFAULT_LENGTH];
-                strncpy(ticket_id, ++data, ticket_syntax_len);
-                ticket_id[ticket_syntax_len] = '\0';
-                o("<a href=\"%s/ticket/%s\">#%s</a>", cgiScriptName, ticket_id, ticket_id);
+                char display_text[DEFAULT_LENGTH];
+                if (strlen(project_id) == 0) {
+                    strcpy(project_id, g_project_name_4_url);
+                    strcpy(display_text, ticket_id);
+                } else {
+                    sprintf(display_text, "%s:%s", project_id, ticket_id);
+                }
+                o("<a href=\"%s/%s/ticket/%s\">#%s</a>",
+                        cgiScriptName, 
+                        project_id, 
+                        ticket_id, 
+                        display_text);
                 data += ticket_syntax_len - 1;
-                len -= ticket_syntax_len;
+                len -= ticket_syntax_len - 1;
             }
         } else if (len > 2 && *data == '>' &&
                 *(data + 1) == '|' &&
@@ -239,7 +400,7 @@ void hmail(char *s)
 void u(char* str)
 {
     char buf[DEFAULT_LENGTH];
-    url_encode(str, buf, DEFAULT_LENGTH);
+    url_encode((unsigned char*)str, (unsigned char*)buf, DEFAULT_LENGTH);
     fprintf(cgiOut, "%s", buf);
 }
 unsigned long url_encode(unsigned char* csource, unsigned char* cbuffer, unsigned long lbuffersize)
@@ -250,27 +411,24 @@ unsigned long url_encode(unsigned char* csource, unsigned char* cbuffer, unsigne
     unsigned char ctemp[4];
     unsigned long lresultcount = 0;
 
-    llength = (unsigned long)strlen(csource);
+    llength = (unsigned long)strlen((char*)csource);
     if(!llength) { return lresultcount; }
     if(lbuffersize < (llength * 3 + 1)) { return lresultcount; }
 
+    d("encoding: %s\n", csource);
     while(1) {
         cbyte = *(csource + lcount);
         if( ((cbyte >= 0x81) && (cbyte <= 0x9F)) ||
                 ((cbyte >= 0xE0) && (cbyte <= 0xEF)) ) {            /* Shift-JIS 2 バイト文字だった場合 */
-            sprintf(ctemp, "%%%02X", cbyte);                        /* URL エンコード(上位バイト) */
-            strncpy(cbuffer + lresultcount, ctemp, 4);
+            sprintf((char*)ctemp, "%%%02X", cbyte);                        /* URL エンコード(上位バイト) */
+            strncpy((char*)(cbuffer + lresultcount), (char*)ctemp, 4);
             lcount++;
             lresultcount += 3;
             if(lcount == llength) { break; }                        /* 文字列の終端に達した場合、ループを抜ける */
-            sprintf(ctemp, "%%%02X", *(csource + lcount));          /* URL エンコード(下位バイト) */
-            strncpy(cbuffer + lresultcount, ctemp, 4);
+            sprintf((char*)ctemp, "%%%02X", *(csource + lcount));          /* URL エンコード(下位バイト) */
+            strncpy((char*)cbuffer + lresultcount, (char*)ctemp, 4);
             lcount++;
             lresultcount += 3;
-        } else if(cbyte == 0x20) {                                  /* 1 バイト半角スペース(" ")だった場合 */
-            strncpy(cbuffer + lresultcount, "+", 2);
-            lcount++;
-            lresultcount++;
         } else if( ((cbyte >= 0x40) && (cbyte <= 0x5A)) ||          /* @A-Z */
                 ((cbyte >= 0x61) && (cbyte <= 0x7A)) ||             /* a-z */
                 ((cbyte >= 0x30) && (cbyte <= 0x39)) ||             /* 0-9 */
@@ -278,17 +436,18 @@ unsigned long url_encode(unsigned char* csource, unsigned char* cbuffer, unsigne
                 (cbyte == 0x2D) ||                                  /* "-" */
                 (cbyte == 0x2E) ||                                  /* "." */
                 (cbyte == 0x5F) ) {                                 /* "_" */ /* 無変換文字だった場合 */
-            strncpy(cbuffer + lresultcount, csource + lcount, 2);
+            strncpy((char*)cbuffer + lresultcount, (char*)csource + lcount, 2);
             lcount++;
             lresultcount++;
         } else {                                                    /* その他の文字の場合 */
-            sprintf(ctemp, "%%%02X", cbyte);                        /* URL エンコード */
-            strncpy(cbuffer + lresultcount, ctemp, 4);
+            sprintf((char*)ctemp, "%%%02X", cbyte);                        /* URL エンコード */
+            strncpy((char*)cbuffer + lresultcount, (char*)ctemp, 4);
             lcount++;
             lresultcount += 3;
         }
         if(lcount == llength) { break; }
     }
+    d("encoded: %s\n", cbuffer);
     return lresultcount;                                            /* cbuffer に書き込んだ文字列のサイズを返す */
 }
 char* get_filename_without_path(char* path)
@@ -351,7 +510,7 @@ ElementFile* get_upload_content(const int element_id)
             p++;
         }
     }
-    content->blob = buffer_org;
+    content->content = buffer_org;
     cgiFormFileClose(file);
     return content;
 }
@@ -374,7 +533,7 @@ static void enclode_char(unsigned long bb, int srclen, unsigned char *dist, int
 
 void base64_encode(const unsigned char *src, unsigned char *dist)
 {
-    unsigned char *p = (char *)src;
+    unsigned char *p = (unsigned char*)src;
     unsigned long bb = (unsigned long)0;
     int i = 0, j = 0;
 
@@ -394,6 +553,19 @@ void base64_encode(const unsigned char *src, unsigned char *dist)
     }
     if (i) enclode_char(bb, i - 1, dist, j);
 }
+void page_not_found()
+{
+    o("Status: 404 Not Found.\r\n");
+    o("Content-Type: text/plain; charset=utf-8;\r\n\r\n");
+    o("404 Not Found.\r\n");
+}
+void redirect_raw(const char* path)
+{
+    char redirecturi[DEFAULT_LENGTH];
+    sprintf(redirecturi, "%s/%s", cgiScriptName, path);
+    o("Status: 302 Temporary Redirection\r\n");
+    cgiHeaderLocation(redirecturi);
+}
 void redirect(const char* path, const char* message)
 {
     char redirecturi[DEFAULT_LENGTH];
@@ -404,11 +576,11 @@ void redirect(const char* path, const char* message)
     strcpy(uri, path);
     if (message) {
         strcpy(param, message);
-        url_encode(param, parambuf, DEFAULT_LENGTH);
+        url_encode((unsigned char*)param, (unsigned char*)parambuf, DEFAULT_LENGTH);
         strcat(uri, "?message=");
         strcat(uri, parambuf);
     }
-    sprintf(redirecturi, "%s%s", cgiScriptName, uri);
+    sprintf(redirecturi, "%s/%s%s", cgiScriptName, g_project_name, uri);
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
@@ -423,7 +595,7 @@ void redirect_with_hook_messages(const char* path, const char* message, List* re
     strcpy(uri, path);
     if (message) {
         strcpy(param, message);
-        url_encode(param, parambuf, DEFAULT_LENGTH);
+        url_encode((unsigned char*)param, (unsigned char*)parambuf, DEFAULT_LENGTH);
         strcat(uri, "?message=");
         strcat(uri, parambuf);
     }
@@ -432,7 +604,7 @@ void redirect_with_hook_messages(const char* path, const char* message, List* re
         foreach (it, results) {
             HOOK_RESULT* result = it->element;
             strcpy(param, result->message);
-            url_encode(param, parambuf, DEFAULT_LENGTH);
+            url_encode((unsigned char*)param, (unsigned char*)parambuf, DEFAULT_LENGTH);
             if (!message && i == 0)
                 strcat(uri, "?");
             else
@@ -442,7 +614,7 @@ void redirect_with_hook_messages(const char* path, const char* message, List* re
             i++;
         }
     }
-    sprintf(redirecturi, "%s%s", cgiScriptName, uri);
+    sprintf(redirecturi, "%s/%s%s", cgiScriptName, g_project_name, uri);
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
@@ -465,6 +637,7 @@ void csv_field(char* src)
     size_t src_size, dist_size, ret_len;
     char* dist_a;
     char* dist_p;
+    bool has_error = false;
 
     src_size = strlen(src) + 1;
     dist_size = src_size;  /* UTF-8からCP932なので、長さが短かくなることはない。そのためdist_sizeにも同じ長さを指定する。*/
@@ -472,8 +645,12 @@ void csv_field(char* src)
     /* 文字コード変換処理 *          */
     ic = iconv_open("CP932", "UTF-8");
     ret_len = iconv(ic, &src, &src_size, &dist_p, &dist_size);
+    if (ret_len == (size_t)-1) {
+        d("iconv failed: %s", strerror(errno));
+        has_error = true;
+    }
     o("\"");
-    if (ret_len >= 0) csv_escape(dist_a, strlen(dist_a));
+    if (!has_error) csv_escape(dist_a, strlen(dist_a));
     o("\"");
     iconv_close(ic);
     xfree(dist_a);
@@ -504,7 +681,7 @@ void css_field(char* str)
     }
         
     memset(dist, 0, DEFAULT_LENGTH);
-    base64_encode(src, dist);
+    base64_encode((unsigned char*)src, (unsigned char*)dist);
     cgiCssClassName(dist, strlen(dist));
 }
 String* get_base_url(String* buf)
@@ -532,11 +709,127 @@ bool contains(char* const value, const char* name)
 }
 void set_cookie(char* key, char* value)
 {
-    cgiHeaderCookieSetString(key, value, 86400 * 30, "/", cgiServerName);
+    char value_base64[DEFAULT_LENGTH];
+    memset(value_base64, 0, DEFAULT_LENGTH);
+    base64_encode((unsigned char*)value, (unsigned char*)value_base64);
+    cgiHeaderCookieSetString(key, value_base64, 86400 * 30, cgiScriptName, cgiServerName);
 }
 void clear_cookie(char* key)
 {
-    cgiHeaderCookieSetString(key, "", 0, "/", cgiServerName);
+    cgiHeaderCookieSetString(key, "", 0, cgiScriptName, cgiServerName);
+}
+void get_cookie_string(char* key, char* buf)
+{
+    char value[DEFAULT_LENGTH];
+    char value_base64[DEFAULT_LENGTH];
+    cgiCookieString(key, value, DEFAULT_LENGTH);
+    memset(value_base64, 0, DEFAULT_LENGTH);
+    base64_decode((unsigned char*)value, (unsigned char*)value_base64);
+    strcpy(buf, value_base64);
+}
+typedef union {
+    unsigned int    data;
+    unsigned char   xyz[4];
+} Union_data_t;
+
+static long codetovalue(unsigned char c)
+{
+    if( (c >= (unsigned char)'A') && (c <= (unsigned char)'Z') ) {
+        return (long)(c - (unsigned char)'A');
+    }
+    else if( (c >= (unsigned char)'a') && (c <= (unsigned char)'z') ) {
+        return ((long)(c - (unsigned char)'a') +26);
+    }
+    else if( (c >= (unsigned char)'0') && (c <= (unsigned char)'9') ) {
+        return ((long)(c - (unsigned char)'0') +52);
+    }
+    else if( (unsigned char)'+' == c ) {
+        return (long)62;
+    }
+    else if( (unsigned char)'/' == c ) {
+        return (long)63;
+    }
+    else if( (unsigned char)'=' == c ) {
+        return (long)0;
+    }
+    else {
+        return -1;
+    }
 }
 
+static int decode_str(int enc_ptr, const unsigned char *src, unsigned char *dest)
+{
+    int i, j;
+    unsigned long base64 = 0;
+    unsigned char x;
+    Union_data_t bb;
+
+    for (i = enc_ptr; i < enc_ptr + 4; i++) {
+        if (src[i] == '\0') return -1; /* '\0'が出現したら終了。 */
+        x = codetovalue(src[i]);
+        base64 |= x;
+        if ((i - enc_ptr) != 3) base64 <<= 6;
+    }
+    base64 <<= 8;
+    bb.data = base64;
+
+    for(j = 0, i = 3; i >= 1; i--) {
+        dest[j++] = bb.xyz[i];
+    }
+    return 0;
+}
+
+int base64_decode(const unsigned char *src, unsigned char *dest)
+{
+    int i = 0;
+    int srclen;
+    char tmp[4];
+
+    /* NULL pointer */
+    if (src == NULL) return -1;
+
+    srclen = strlen((char*)src);
+
+    if (!srclen % 4) return -2;
+
+    while(srclen) {
+        memset(tmp, 0, sizeof(tmp));
+        if (decode_str(i, src, (unsigned char*)tmp) != 0)
+            return -1; /* 失敗した場合は、終了 */
+        strcat((char*)dest, tmp);
+        i += 4;
+        srclen -= 4;
+    }
+
+    return 0;
+}
+void set_date_string(char* buf)
+{
+    struct timeval tv;
+    struct tm *tp;
+
+    gettimeofday(&tv, NULL);
+    tp = localtime(&tv.tv_sec);
+    sprintf(buf, "%04d-%02d-%02d %02d:%02d:%02d",
+            tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday,
+            tp->tm_hour, tp->tm_min, tp->tm_sec);
+}
+/*
+ * 引数の文字列から拡張子部分のポインタを返す。
+ */
+char* get_ext(char* path)
+{
+    char* p = path;
+    /* 後ろから探して.があったら、そこの文字配列のポインタを返却する。 */
+    for (p += strlen(path); p > path; p--) {
+        switch (*p) {
+            case '.':
+                return (char*)p + 1;
+            case '\\':
+            case '/':
+                return "";
+        }
+    }
+    return "";
+}
 /* vim: set ts=4 sw=4 sts=4 expandtab fenc=utf-8: */
