@@ -4,12 +4,12 @@
 #include <cgic.h>
 #include "wiki.h"
 #include "util.h"
-#include "db.h"
+#include "db_project.h"
 
 #define MAX_WIDTH 1000
 
 void buf_clear();
-void buf_add(int, char*);
+void buf_add(unsigned int, char*);
 void buf_flush();
 void text_out(char*);
 void element_out_without_content(char*);
@@ -28,7 +28,7 @@ void buf_clear() {
         strcpy(text[i], "");
     }
 }
-void buf_add(int type, char* str)
+void buf_add(unsigned int type, char* str)
 {
     int i;
     if (buf_type != type)
@@ -94,87 +94,85 @@ void element_out(char* tag_name, char* content)
     buf_out(content);
     printf("</%s>\n", tag_name);
 }
-enum LINE_MODE {
+typedef enum LINE_MODE {
     LINE_MODE_NORMAL,
     LINE_MODE_PRE
-};
-void wiki_out(char* page_name)
+} LineMode;
+void wiki_out(Database* db, char* page_name)
 {
-    Wiki* wiki_a = xalloc(sizeof(Wiki));
-    char line[MAX_WIDTH];
+    Wiki* wiki_a = wiki_new();
+    char* line_a;
     char* c;
     char* p;
-    int mode = LINE_MODE_NORMAL;
+    LineMode mode = LINE_MODE_NORMAL;
 
-    wiki_a = db_get_newest_wiki(page_name, wiki_a);
+    wiki_a = db_get_newest_wiki(db, page_name, wiki_a);
     p = wiki_a->content;
+    /* FIXME 非効率だけど、安全のために、1行分のバッファに全文の長さを確保する。 */
+    line_a = xalloc(sizeof(char) * strlen(wiki_a->content));
     buf_clear();
     while ((strlen(p) != 0)) {
         if ((c = strchr(p, '\n')) != NULL) {
             int len = c - p;
-            strncpy(line, p, len);
-            line[len] = '\0';
+            strncpy(line_a, p, len);
+            line_a[len] = '\0';
             p += len + 1;
         } else {
-            strcpy(line, p);
+            strcpy(line_a, p);
             p += strlen(p);
         }
-        if (mode == LINE_MODE_PRE && strncmp(line, "|<", strlen("|<")) != 0) {
+        if (mode == LINE_MODE_PRE && strncmp(line_a, "|<", strlen("|<")) != 0) {
             /* LINE_MODE_PRE中は、成形済ブロック */
-            buf_add(TYPE_PRE, line);
-        } else if (mode == LINE_MODE_NORMAL && strncmp(line, ">|", strlen(">|")) == 0) {
+            buf_add(TYPE_PRE, line_a);
+        } else if (mode == LINE_MODE_NORMAL && strncmp(line_a, ">|", strlen(">|")) == 0) {
             /* 成形済ブロック終了 */
             buf_flush();
             mode = LINE_MODE_PRE;
-        } else if (mode == LINE_MODE_PRE && strncmp(line, "|<", strlen("|<")) == 0) {
+        } else if (mode == LINE_MODE_PRE && strncmp(line_a, "|<", strlen("|<")) == 0) {
             /* 成形済ブロック開始 */
             buf_flush();
             mode = LINE_MODE_NORMAL;
-        } else if (strncmp(line, "****", strlen("****")) == 0) {
+        } else if (strncmp(line_a, "****", strlen("****")) == 0) {
             buf_flush();
-            element_out("h6", line + strlen("****"));
-        } else if (strncmp(line, "***", strlen("***")) == 0) {
+            element_out("h6", line_a + strlen("****"));
+        } else if (strncmp(line_a, "***", strlen("***")) == 0) {
             buf_flush();
-            element_out("h5", line + strlen("***"));
-        } else if (strncmp(line, "**", strlen("**")) == 0) {
+            element_out("h5", line_a + strlen("***"));
+        } else if (strncmp(line_a, "**", strlen("**")) == 0) {
             buf_flush();
-            element_out("h4", line + strlen("**"));
-        } else if (strncmp(line, "*", strlen("*")) == 0) {
+            element_out("h4", line_a + strlen("**"));
+        } else if (strncmp(line_a, "*", strlen("*")) == 0) {
             buf_flush();
-            element_out("h3", line + strlen("*"));
-        } else if (strncmp(line, "----", strlen("----")) == 0) {
+            element_out("h3", line_a + strlen("*"));
+        } else if (strncmp(line_a, "----", strlen("----")) == 0) {
             buf_flush();
             element_out_without_content("hr");
-        } else if (strncmp(line, "-", strlen("-")) == 0) {
-            buf_add(TYPE_LI, line + strlen("-"));
-/*         } else if (strncmp(line, " ", strlen(" ")) == 0) { */
-/*             buf_add(TYPE_PRE, line); */
-        } else if (strncmp(line, "\n", strlen("\n")) == 0) {
+        } else if (strncmp(line_a, "-", strlen("-")) == 0) {
+            buf_add(TYPE_LI, line_a + strlen("-"));
+        } else if (strncmp(line_a, "\n", strlen("\n")) == 0) {
             buf_flush();
         } else {
-            buf_add(TYPE_TEXT, line);
+            buf_add(TYPE_TEXT, line_a);
         }
     }
     buf_flush();
-    xfree(wiki_a->content);
-    xfree(wiki_a);
+    xfree(line_a);
+    wiki_free(wiki_a);
 }
-void wiki_content_out(char* page_name)
+void wiki_content_out(Database* db, char* page_name)
 {
-    Wiki* wiki_a = xalloc(sizeof(Wiki));
-    wiki_a = db_get_newest_wiki(page_name, wiki_a);
+    Wiki* wiki_a = wiki_new();
+    wiki_a = db_get_newest_wiki(db, page_name, wiki_a);
     h(wiki_a->content);
-    xfree(wiki_a->content);
-    xfree(wiki_a);
+    wiki_free(wiki_a);
 }
-void wiki_save(char* page_name, char* content)
+void wiki_save(Database* db, char* page_name, char* content)
 {
-    Wiki* wiki_a = xalloc(sizeof(Wiki));
+    Wiki* wiki_a = wiki_new();
     wiki_a->content = xalloc(sizeof(char) * strlen(content) + 1);
     strcpy(wiki_a->name, page_name);
     strcpy(wiki_a->content, content);
-    db_register_wiki(wiki_a);
-    xfree(wiki_a->content);
-    xfree(wiki_a);
+    db_register_wiki(db, wiki_a);
+    wiki_free(wiki_a);
 }
 /* vim: set ts=4 sw=4 sts=4 expandtab fenc=utf-8: */
